/**
 * @class {util4js.Dictionary}
 * @extends {util4js.Object}
 * @implements {util4js.IDictionary}
 */
var Dictionary = $class({

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @constructor {util4js.Dictionary}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = global.Object.create(null);
    this.__length = 0;
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i < items.length; i++) {
          if ("key" in items[i] && "value" in items[i]) {
            this.add(items[i].key, items[i].value);
          } else {
            throw new ArgumentException({
              paramName: "items"
            }); 
          }
        }
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      }
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Dictionary";
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new DictionaryIterator(this.__items);
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__length;
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @methos clear
   */
  clear: function () {
    this.__length = 0;
    this.__items = {};
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method add
   * @param {Object} key
   * @param {Object} value
   */
  add: function (key, value) {
    var hashCode = this.__getKeyHashCode(key);
    if (!this.contains(hashCode)) {
      this.__items[hashCode] = new KeyValuePair(key, value);
      this.__length++;
    } else {
      throw new ArgumentException({ 
        paramName: "key",
        message: "An entry with dublicated key can't be added to the dictionary"
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method remove
   * @param {Object} key
   * @returns {Boolean}
   */
  remove: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      delete this.__items[hashCode];
      this.__length--;
      return true;
    }
    return false;
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method contains
   * @param {Object} key
   * @returns {Boolean}
   */
  contains: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    return (hashCode in this.__items);
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method get
   * @param {Object} key
   * @returns {Object}
   */
  get: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      return this.__items[hashCode].value;
    } else {
      throw new ArgumentException({
        paramName: "key",
        message: "Key doesn't exists in the dictionary"
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method set
   * @param {Object} key
   * @param {Object} value
   */
  set: function (key, value) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      this.__items[hashCode].value = value;
    } else {
      throw new ArgumentException({ 
        paramName: "key",
        message: "Key doesn't exists in the dictionary" 
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @private
   * @method __getKeyHashCode
   * @param {Object} key
   */
  __getKeyHashCode: function (key) {
    if (!key) {
      throw new ArgumentNullException({ 
        paramName: "key"
      });
    }
    if ($is(key, Object)) {
      return key.getHashCode();
    } else if (typeof key === "object") {
      throw new ArgumentException({
        paramName: "key",
        message: "Type is invalid"
      });
    }
    return key;
  }

}, Object, IDictionary);

exports.Dictionary = Dictionary;

